"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const categories_1 = __importDefault(require("./categories"));
const algorithmCategories_1 = __importDefault(require("./algorithmCategories"));
const staticAnalysisCommon_1 = require("./staticAnalysisCommon");
class etherTransferInLoop {
    constructor() {
        this.relevantNodes = [];
        this.name = 'Ether transfer in a loop: ';
        this.description = 'Avoid transferring Ether to multiple addresses in a loop';
        this.category = categories_1.default.GAS;
        this.algorithm = algorithmCategories_1.default.EXACT;
    }
    visit(node) {
        if (staticAnalysisCommon_1.isLoop(node)) {
            let transferNodes = [];
            const loopBlockStartIndex = staticAnalysisCommon_1.getLoopBlockStartIndex(node);
            if (loopBlockStartIndex && node.children && staticAnalysisCommon_1.isBlock(node.children[loopBlockStartIndex])) {
                const childrenNodes = node.children[loopBlockStartIndex].children;
                if (childrenNodes)
                    transferNodes = childrenNodes.filter(child => (staticAnalysisCommon_1.isExpressionStatement(child) &&
                        child.children &&
                        child.children[0].name === 'FunctionCall' &&
                        child.children[0].children &&
                        staticAnalysisCommon_1.isTransfer(child.children[0].children[0])));
                if (transferNodes.length > 0) {
                    this.relevantNodes.push(...transferNodes);
                }
            }
        }
    }
    report(compilationResults) {
        return this.relevantNodes.map((node) => {
            return {
                warning: 'Ether payout should not be done in a loop: Due to the block gas limit, transactions can only consume a certain amount of gas. The number of iterations in a loop can grow beyond the block gas limit which can cause the complete contract to be stalled at a certain point. If required then make sure that number of iterations are low and you trust each address involved.',
                location: node.src,
                more: 'https://solidity.readthedocs.io/en/latest/security-considerations.html#gas-limit-and-loops'
            };
        });
    }
}
exports.default = etherTransferInLoop;
//# sourceMappingURL=etherTransferInLoop.js.map