"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const categories_1 = __importDefault(require("./categories"));
const algorithmCategories_1 = __importDefault(require("./algorithmCategories"));
class gasCosts {
    constructor() {
        this.name = 'Gas costs: ';
        this.description = 'Warn if the gas requirements of functions are too high.';
        this.category = categories_1.default.GAS;
        this.algorithm = algorithmCategories_1.default.EXACT;
    }
    /**
      * call the given @arg cb (function) for all the contracts. Uses last compilation result
      * stop visiting when cb return true
      * @param {Function} cb    - callback
      */
    // @TODO has been copied from remix-ide repo ! should fix that soon !
    visitContracts(contracts, cb) {
        for (let file in contracts) {
            for (let name in contracts[file]) {
                if (cb({ name: name, object: contracts[file][name], file: file }))
                    return;
            }
        }
    }
    report(compilationResults) {
        const report = [];
        this.visitContracts(compilationResults.contracts, (contract) => {
            if (!contract.object.evm.gasEstimates ||
                !contract.object.evm.gasEstimates.external) {
                return;
            }
            const fallback = contract.object.evm.gasEstimates.external[''];
            if (fallback !== undefined) {
                if (fallback === null || fallback >= 2100 || fallback === 'infinite') {
                    report.push({
                        warning: `Fallback function of contract ${contract.name} requires too much gas (${fallback}). 
            If the fallback function requires more than 2300 gas, the contract cannot receive Ether.`
                    });
                }
            }
            for (var functionName in contract.object.evm.gasEstimates.external) {
                if (functionName === '') {
                    continue;
                }
                const gas = contract.object.evm.gasEstimates.external[functionName];
                const gasString = gas === null ? 'unknown or not constant' : 'high: ' + gas;
                if (gas === null || gas >= 3000000 || gas === 'infinite') {
                    report.push({
                        warning: `Gas requirement of function ${contract.name}.${functionName} ${gasString}. 
            If the gas requirement of a function is higher than the block gas limit, it cannot be executed.
            Please avoid loops in your functions or actions that modify large areas of storage
            (this includes clearing or copying arrays in storage)`
                    });
                }
            }
        });
        return report;
    }
}
exports.default = gasCosts;
//# sourceMappingURL=gasCosts.js.map