declare const nodeTypes: {
    IDENTIFIER: string;
    MEMBERACCESS: string;
    FUNCTIONCALL: string;
    FUNCTIONDEFINITION: string;
    VARIABLEDECLARATION: string;
    ASSIGNMENT: string;
    CONTRACTDEFINITION: string;
    UNARYOPERATION: string;
    BINARYOPERATION: string;
    EXPRESSIONSTATEMENT: string;
    MODIFIERDEFINITION: string;
    MODIFIERINVOCATION: string;
    INHERITANCESPECIFIER: string;
    USERDEFINEDTYPENAME: string;
    INLINEASSEMBLY: string;
    BLOCK: string;
    NEWEXPRESSION: string;
    RETURN: string;
    IFSTATEMENT: string;
    FORSTATEMENT: string;
    WHILESTATEMENT: string;
    DOWHILESTATEMENT: string;
    ELEMENTARYTYPENAMEEXPRESSION: string;
};
declare const basicTypes: {
    UINT: string;
    BOOL: string;
    ADDRESS: string;
    PAYABLE_ADDRESS: string;
    BYTES32: string;
    STRING_MEM: string;
    BYTES_MEM: string;
    BYTES4: string;
};
declare const basicFunctionTypes: {
    SEND: string;
    CALL: string;
    'CALL-v0.5': string;
    DELEGATECALL: string;
    'DELEGATECALL-v0.5': string;
    TRANSFER: string;
};
declare const lowLevelCallTypes: {
    CALL: {
        ident: string;
        type: string;
    };
    'CALL-v0.5': {
        ident: string;
        type: string;
    };
    CALLCODE: {
        ident: string;
        type: string;
    };
    DELEGATECALL: {
        ident: string;
        type: string;
    };
    'DELEGATECALL-v0.5': {
        ident: string;
        type: string;
    };
    SEND: {
        ident: string;
        type: string;
    };
    TRANSFER: {
        ident: string;
        type: string;
    };
};
declare const specialVariables: {
    BLOCKTIMESTAMP: {
        obj: string;
        member: string;
        type: string;
    };
    BLOCKHASH: {
        obj: string;
        member: string;
        type: string;
    };
};
declare function getType(node: any): any;
/**
 * Returns the type parameter of function call AST nodes. Throws if not a function call node
 * @func {ASTNode} Function call node
 * @return {string} type of function call
 */
declare function getFunctionCallType(func: any): any;
/**
 * Get the variable name written to by a effect node, except for inline assembly because there is no information to find out where we write to. Trows if not a effect node or is inlineassmbly.
 * Example: x = 10; => x
 * @effectNode {ASTNode} Function call node
 * @return {string} variable name written to
 */
declare function getEffectedVariableName(effectNode: any): any;
/**
 * Returns the identifier of a local call, Throws on wrong node.
 * Example: f(103) => f
 * @localCallNode {ASTNode} Function call node
 * @return {string} name of the function called
 */
declare function getLocalCallName(localCallNode: any): any;
/**
 * Returns the identifier of a this local call, Throws on wrong node.
 * Example: this.f(103) => f
 * @localCallNode {ASTNode} Function call node
 * @return {string} name of the function called
 */
declare function getThisLocalCallName(localCallNode: any): any;
/**
 * Returns the identifier of a super local call, Throws on wrong node.
 * Example: super.f(103) => f
 * @localCallNode {ASTNode} Function call node
 * @return {string} name of the function called
 */
declare function getSuperLocalCallName(localCallNode: any): any;
/**
 * Returns the contract type of a external direct call, Throws on wrong node.
 * Example:
 *  foo x = foo(0xdeadbeef...);
 *  x.f(103) => foo
 * @extDirectCall {ASTNode} Function call node
 * @return {string} name of the contract the function is defined in
 */
declare function getExternalDirectCallContractName(extDirectCall: any): any;
/**
 * Returns the name of the contract of a this local call (current contract), Throws on wrong node.
 * Example:
 * Contract foo {
 *    ...
 *    this.f(103) => foo
 *    ...
 * @thisLocalCall {ASTNode} Function call node
 * @return {string} name of the contract the function is defined in
 */
declare function getThisLocalCallContractName(thisLocalCall: any): any;
/**
 * Returns the function identifier of a external direct call, Throws on wrong node.
 * Example:
 *  foo x = foo(0xdeadbeef...);
 *  x.f(103) => f
 * @extDirectCall {ASTNode} Function call node
 * @return {string} name of the function called
 */
declare function getExternalDirectCallMemberName(extDirectCall: any): any;
/**
 * Returns the name of a contract, Throws on wrong node.
 * Example:
 *   Contract foo { => foo
 * @contract {ASTNode} Contract Definition node
 * @return {string} name of a contract defined
 */
declare function getContractName(contract: any): any;
/**
 * Returns the name of a function definition, Throws on wrong node.
 * Example:
 *   func foo(uint bla) { => foo
 * @funcDef {ASTNode} Function Definition node
 * @return {string} name of a function defined
 */
declare function getFunctionDefinitionName(funcDef: any): any;
/**
 * Returns the identifier of an inheritance specifier, Throws on wrong node.
 * Example:
 *   contract KingOfTheEtherThrone is b { => b
 * @func {ASTNode} Inheritance specifier
 * @return {string} name of contract inherited from
 */
declare function getInheritsFromName(inheritsNode: any): any;
/**
 * Returns the identifier of a variable definition, Throws on wrong node.
 * Example:
 *   var x = 10; => x
 * @varDeclNode {ASTNode} Variable declaration node
 * @return {string} variable name
 */
declare function getDeclaredVariableName(varDeclNode: any): any;
/**
 * Returns the type of a variable definition, Throws on wrong node.
 * Example:
 *   var x = 10; => x
 * @varDeclNode {ASTNode} Variable declaration node
 * @return {string} variable type
 */
declare function getDeclaredVariableType(varDeclNode: any): any;
/**
 * Returns state variable declaration nodes for a contract, Throws on wrong node.
 * Example:
 * contract foo {
 *     ...
 *   var y = true;
 *   var x = 10; => [y,x]
 * @contractNode {ASTNode} Contract Definition node
 * @return {list variable declaration} state variable node list
 */
declare function getStateVariableDeclarationsFormContractNode(contractNode: any): any;
/**
 * Returns parameter node for a function or modifier definition, Throws on wrong node.
 * Example:
 * function bar(uint a, uint b) => uint a, uint b
 * @funcNode {ASTNode} Contract Definition node
 * @return {parameterlist node} parameterlist node
 */
declare function getFunctionOrModifierDefinitionParameterPart(funcNode: any): any;
/**
 * Returns return parameter node for a function or modifier definition, Throws on wrong node.
 * Example:
 * function bar(uint a, uint b) returns (bool a, bool b) => bool a, bool b
 * @funcNode {ASTNode} Contract Definition node
 * @return {parameterlist node} parameterlist node
 */
declare function getFunctionOrModifierDefinitionReturnParameterPart(funcNode: any): any;
/**
 * Extracts the parameter types for a function type signature
 * Example:
 * function(uint a, uint b) returns (bool) => uint a, uint b
 * @func {ASTNode} function call node
 * @return {string} parameter signature
 */
declare function getFunctionCallTypeParameterType(func: any): string | undefined;
/**
 * Returns the name of the library called, Throws on wrong node.
 * Example:
 *  library set{...}
 *  contract foo {
 *    ...
 *    function () { set.union() => set}
 * @funcCall {ASTNode} function call node
 * @return {string} name of the lib defined
 */
declare function getLibraryCallContractName(funcCall: any): string | undefined;
/**
 * Returns the name of the function of a library call, Throws on wrong node.
 * Example:
 *  library set{...}
 *  contract foo {
 *    ...
 *    function () { set.union() => uinion}
 * @func {ASTNode} function call node
 * @return {string} name of function called on the library
 */
declare function getLibraryCallMemberName(funcCall: any): any;
/**
 * Returns full qualified name for a function call, Throws on wrong node.
 * Example:
 *  contract foo {
 *    ...
 *    function bar(uint b) { }
 *    function baz() {
 *      bar(10) => foo.bar(uint)
 * @func {ASTNode} function call node
 * @func {ASTNode} contract defintion
 * @return {string} full qualified identifier for the function call
 */
declare function getFullQualifiedFunctionCallIdent(contract: any, func: any): string;
declare function getFullQuallyfiedFuncDefinitionIdent(contract: any, func: any, paramTypes: any): string;
declare function getUnAssignedTopLevelBinOps(subScope: any): any;
declare function getLoopBlockStartIndex(node: any): 1 | 3 | undefined;
declare function isFunctionDefinition(node: any): boolean;
declare function isStatement(node: any): boolean;
declare function isBlock(node: any): boolean;
declare function isModifierDefinition(node: any): boolean;
declare function isModifierInvocation(node: any): boolean;
declare function isVariableDeclaration(node: any): boolean;
declare function isReturn(node: any): boolean;
declare function isInheritanceSpecifier(node: any): boolean;
declare function isAssignment(node: any): boolean;
declare function isContractDefinition(node: any): boolean;
declare function isInlineAssembly(node: any): boolean;
declare function isNewExpression(node: any): boolean;
/**
 * True if is Expression
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isExpressionStatement(node: any): boolean;
/**
 * True if is binaryop
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isBinaryOperation(node: any): boolean;
/**
 * True if function defintion has function body
 * @funcNode {ASTNode} function defintion node
 * @return {bool}
 */
declare function hasFunctionBody(funcNode: any): boolean;
/**
 * True if node is a delete instruction of a dynamic array
 * @node {ASTNode} node to check for
 * @return {bool}
 */
declare function isDeleteOfDynamicArray(node: any): any;
/**
 * True if node is node is a ref to a dynamic array
 * @node {ASTNode} node to check for
 * @return {bool}
 */
declare function isDynamicArrayAccess(node: any): any;
/**
 * True if node accesses 'length' member of dynamic array
 * @node {ASTNode} node to check for
 * @return {bool}
 */
declare function isDynamicArrayLengthAccess(node: any): boolean;
/**
 * True if node is a delete instruction for an element from a dynamic array
 * @node {ASTNode} node to check for
 * @return {bool}
 */
declare function isDeleteFromDynamicArray(node: any): boolean;
/**
 * True if node is the access of an index
 * @node {ASTNode} node to check for
 * @return {bool}
 */
declare function isIndexAccess(node: any): boolean;
/**
 * True if node is the access of a mapping index
 * @node {ASTNode} node to check for
 * @return {bool}
 */
declare function isMappingIndexAccess(node: any): any;
/**
 * True if call to code within the current contracts context including (delegate) library call
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isLocalCallGraphRelevantNode(node: any): boolean;
/**
 * True if is builtin function like assert, sha3, erecover, ...
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isBuiltinFunctionCall(node: any): boolean;
/**
 * True if is builtin function like assert, sha3, erecover, ...
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isAbiNamespaceCall(node: any): boolean;
/**
 * True if node is a call to selfdestruct
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isSelfdestructCall(node: any): boolean;
/**
 * True if node is a call to builtin assert(bool)
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isAssertCall(node: any): boolean;
/**
 * True if node is a call to builtin require(bool)
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isRequireCall(node: any): boolean;
/**
 * True if is storage variable declaration
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isStorageVariableDeclaration(node: any): boolean;
/**
 * True if is interaction with external contract (change in context, no delegate calls) (send, call of other contracts)
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isInteraction(node: any): boolean;
/**
 * True if node changes state of a variable or is inline assembly (does not include check if it is a global state change, on a state variable)
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isEffect(node: any): boolean;
/**
 * True if node changes state of a variable or is inline assembly (Checks if variable is a state variable via provided list)
 * @node {ASTNode} some AstNode
 * @node {list Variable declaration} state variable declaration currently in scope
 * @return {bool}
 */
declare function isWriteOnStateVariable(effectNode: any, stateVariables: any): any;
/**
 * True if there is a variable with name, name in stateVariables
 * @node {ASTNode} some AstNode
 * @node {list Variable declaration} state variable declaration currently in scope
 * @return {bool}
 */
declare function isStateVariable(name: any, stateVariables: any): any;
/**
 * True if is function defintion that is flaged as constant
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isConstantFunction(node: any): boolean;
/**
 * True if is function defintion has payable modifier
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isPayableFunction(node: any): boolean;
/**
 * True if is constructor
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isConstructor(node: any): boolean;
/**
 * True if node is integer division that truncates (not only int literals since those yield a rational value)
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isIntDivision(node: any): boolean;
/**
 * True if is block / SubScope has top level binops (e.g. that are not assigned to anything, most of the time confused compare instead of assign)
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isSubScopeWithTopLevelUnAssignedBinOp(node: any): any;
/**
 * True if unary increment operation
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isPlusPlusUnaryOperation(node: any): boolean;
/**
 * True if unary delete operation
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isDeleteUnaryOperation(node: any): boolean;
/**
 * True if unary decrement operation
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isMinusMinusUnaryOperation(node: any): boolean;
/**
 * True if all functions on a contract are implemented
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isFullyImplementedContract(node: any): boolean;
/**
 * True if it is a library contract defintion
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isLibrary(node: any): boolean;
/**
 * True if it is a local call to non const function
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isCallToNonConstLocalFunction(node: any): boolean;
/**
 * True if it is a call to a library
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isLibraryCall(node: any): boolean;
/**
 * True if it is an external call via defined interface (not low level call)
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isExternalDirectCall(node: any): boolean;
/**
 * True if access to block.timestamp via now alias
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isNowAccess(node: any): boolean;
/**
 * True if access to block.timestamp
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isBlockTimestampAccess(node: any): boolean;
/**
 * True if access to block.blockhash
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isBlockBlockHashAccess(node: any): boolean;
/**
 * True if call to local function via this keyword
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isThisLocalCall(node: any): boolean;
/**
 * True if access to local function via super keyword
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isSuperLocalCall(node: any): boolean;
/**
 * True if call to local function
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isLocalCall(node: any): boolean;
/**
 * True if low level call (send, call, delegatecall, callcode)
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isLowLevelCall(node: any): boolean;
/**
 * True if low level send (solidity >= 0.5)
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isLLSend050(node: any): boolean;
/**
 * True if low level send (solidity < 0.5)
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isLLSend(node: any): boolean;
/**
 * True if low level call
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isLLCall(node: any): boolean;
/**
 * True if low level payable call (solidity >= 0.5)
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isLLCall050(node: any): boolean;
/**
 * True if low level callcode
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isLLCallcode(node: any): boolean;
/**
 * True if low level delegatecall
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isLLDelegatecall(node: any): boolean;
/**
 * True if low level delegatecall (solidity >= 0.5)
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isLLDelegatecall050(node: any): boolean;
/**
 * True if transfer call
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isTransfer(node: any): boolean;
declare function isStringToBytesConversion(node: any): boolean;
declare function isBytesLengthCheck(node: any): boolean;
/**
 * True if it is a loop
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isLoop(node: any): boolean;
/**
 * True if it is a 'for' loop
 * @node {ASTNode} some AstNode
 * @return {bool}
 */
declare function isForLoop(node: any): boolean;
declare function isSpecialVariableAccess(node: any, varType: any): boolean;
declare function nrOfChildren(node: any, nr: any): boolean;
declare function minNrOfChildren(node: any, nr: any): boolean;
declare function expressionType(node: any, typeRegex: any): boolean;
declare function nodeType(node: any, typeRegex: any): boolean;
declare function memName(node: any, memNameRegex: any): boolean;
declare function operator(node: any, opRegex: any): boolean;
/**
 * Builds an function signature as used in the AST of the solc-json AST
 * @param {Array} paramTypes
 *  list of parameter type names
 * @param {Array} returnTypes
 *  list of return type names
 * @return {Boolean} isPayable
 */
declare function buildFunctionSignature(paramTypes: any, returnTypes: any, isPayable: any, additionalMods?: any): string;
declare function buildAbiSignature(funName: any, paramTypes: any): string;
declare const helpers: {
    nrOfChildren: typeof nrOfChildren;
    minNrOfChildren: typeof minNrOfChildren;
    expressionType: typeof expressionType;
    nodeType: typeof nodeType;
    memName: typeof memName;
    operator: typeof operator;
    buildFunctionSignature: typeof buildFunctionSignature;
    buildAbiSignature: typeof buildAbiSignature;
};
export { getType, getThisLocalCallName, getSuperLocalCallName, getFunctionCallType, getContractName, getEffectedVariableName, getDeclaredVariableName, getDeclaredVariableType, getLocalCallName, getInheritsFromName, getExternalDirectCallContractName, getThisLocalCallContractName, getExternalDirectCallMemberName, getFunctionDefinitionName, getFunctionCallTypeParameterType, getLibraryCallContractName, getLibraryCallMemberName, getFullQualifiedFunctionCallIdent, getFullQuallyfiedFuncDefinitionIdent, getStateVariableDeclarationsFormContractNode, getFunctionOrModifierDefinitionParameterPart, getFunctionOrModifierDefinitionReturnParameterPart, getUnAssignedTopLevelBinOps, getLoopBlockStartIndex, isDeleteOfDynamicArray, isDeleteFromDynamicArray, isAbiNamespaceCall, isSpecialVariableAccess, isDynamicArrayAccess, isDynamicArrayLengthAccess, isIndexAccess, isMappingIndexAccess, isSubScopeWithTopLevelUnAssignedBinOp, hasFunctionBody, isInteraction, isEffect, isNowAccess, isBlockTimestampAccess, isBlockBlockHashAccess, isThisLocalCall, isSuperLocalCall, isLibraryCall, isLocalCallGraphRelevantNode, isLocalCall, isWriteOnStateVariable, isStateVariable, isTransfer, isLowLevelCall, isLLCall as isLowLevelCallInst, isLLCall050 as isLowLevelCallInst050, isLLSend050 as isLowLevelSendInst050, isLLDelegatecall050 as isLLDelegatecallInst050, isLLCallcode as isLowLevelCallcodeInst, isLLDelegatecall as isLowLevelDelegatecallInst, isLLSend as isLowLevelSendInst, isExternalDirectCall, isFullyImplementedContract, isLibrary, isCallToNonConstLocalFunction, isPlusPlusUnaryOperation, isMinusMinusUnaryOperation, isBuiltinFunctionCall, isSelfdestructCall, isAssertCall, isRequireCall, isIntDivision, isStringToBytesConversion, isBytesLengthCheck, isLoop, isForLoop, isDeleteUnaryOperation, isFunctionDefinition, isModifierDefinition, isInheritanceSpecifier, isModifierInvocation, isVariableDeclaration, isStorageVariableDeclaration, isAssignment, isContractDefinition, isConstantFunction, isPayableFunction, isConstructor, isInlineAssembly, isNewExpression, isReturn, isStatement, isExpressionStatement, isBlock, isBinaryOperation, nodeTypes, basicTypes, basicFunctionTypes, lowLevelCallTypes, specialVariables, helpers };
