"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const categories_1 = __importDefault(require("./categories"));
const algorithmCategories_1 = __importDefault(require("./algorithmCategories"));
class txOrigin {
    constructor() {
        this.txOriginNodes = [];
        this.name = 'Transaction origin: ';
        this.description = 'Warn if tx.origin is used';
        this.category = categories_1.default.SECURITY;
        this.algorithm = algorithmCategories_1.default.EXACT;
    }
    visit(node) {
        if (node.name === 'MemberAccess' && node.attributes &&
            node.attributes.member_name === 'origin' &&
            (node.attributes.type === 'address' || node.attributes.type === 'address payable') &&
            node.children && node.children.length && node.children[0].attributes &&
            node.children[0].attributes.type === 'tx' &&
            node.children[0].attributes.value === 'tx') {
            this.txOriginNodes.push(node);
        }
    }
    report(compilationResults) {
        return this.txOriginNodes.map((item, i) => {
            return {
                warning: `Use of tx.origin: "tx.origin" is useful only in very exceptional cases. 
                  If you use it for authentication, you usually want to replace it by "msg.sender", because otherwise any contract you call can act on your behalf.`,
                location: item.src
            };
        });
    }
}
exports.default = txOrigin;
//# sourceMappingURL=txOrigin.js.map